@@ -9,14 +9,22 @@ isN0 <- function(x)  is.na(x) | x != 0
 is1  <- function(x) !is.na(x) & x   # also == "isTRUE componentwise"
 
 ##
-all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
-any0 <- function(x) isTRUE(any(x == 0))	      ## ~= anyFalse
+##all0 <- function(x) !any(is.na(x)) && all(!x) ## ~= allFalse
+all0 <- function(x) .Call(R_all0, x)
+
+## any0 <- function(x) isTRUE(any(x == 0))	      ## ~= anyFalse
+any0 <- function(x) .Call(R_any0, x)
+
 ## These work "identically" for  1 ('==' TRUE)  and 0 ('==' FALSE)
 ##	(but give a warning for "double"  1 or 0)
 ## TODO: C versions of these would be faster
 allTrue  <- function(x) all(x)  && !any(is.na(x))
-allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0
-anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
+
+##allFalse <- function(x) !any(x) && !any(is.na(x))## ~= all0, but allFalse(NULL) = TRUE w/warning
+allFalse <- function(x) .Call(R_all0, x)
+
+##anyFalse <- function(x) isTRUE(any(!x))		 ## ~= any0
+anyFalse <- function(x) .Call(R_any0, x)
 
 as1 <- function(x, mod=mode(x))
     switch(mod, "integer"= 1L, "double"=, "numeric"= 1, "logical"= TRUE,
@@ -102,7 +110,8 @@ MatrixClass <- function(cl, cld = getClassDef(cl),
     ## ----------------------------------------------------------------------
     ## Arguments: cl: string, class name
     ##		 cld: its class definition
-    ##	   ...Matrix: if TRUE, the result must be of pattern "...Matrix"
+    ##	   ...Matrix: if TRUE, the result must be of pattern "[dlniz]..Matrix"
+    ##                where the first letter "[dlniz]" denotes the content kind.
     ##	      ..... : other arguments are passed to .selectSuperClasses()
     ## ----------------------------------------------------------------------
     ## Author: Martin Maechler, Date: 24 Mar 2009
@@ -114,7 +123,7 @@ MatrixClass <- function(cl, cld = getClassDef(cl),
 	## else we use 'pkg'
     }
     if(identical(pkg, "Matrix") &&
-       (!...Matrix || identical(1L, grep("^...Matrix$", cl))))
+       (!...Matrix || (cl != "indMatrix" && identical(1L, grep("^[dlniz]..Matrix$", cl)))))
 	cl
     else { ## possibly recursively
 	r <- .selectSuperClasses(cld@contains, dropVirtual = dropVirtual,
@@ -135,26 +144,36 @@ attrSlotNames <- function(m, factors = TRUE) {
 attrSlots <- function(m, factors = TRUE) sapply(attrSlotNames(m, factors=factors),
 			 function(sn) slot(m, sn), simplify = FALSE)
 
+##' @return { NULL | TRUE | character | list(.) }
 attr.all_Mat <- function(target, current,
 			 check.attributes = TRUE, factorsCheck = FALSE, ...) {
     msg <- if(check.attributes)
 	all.equal(attrSlots(target,  factors=factorsCheck),
 		  attrSlots(current, factors=factorsCheck),
-		  check.attributes = TRUE, ...)
+		  check.attributes = TRUE, ...) ## else NULL
     if((c1 <- class(target)) != (c2 <- class(current)))
 	## list(): so we can easily check for this
-	list(c(msg, paste0("class(target) is ", c1, ", current is ", c2)))
+	list(c(if(!isTRUE(msg)) msg, paste0("class(target) is ", c1, ", current is ", c2)))
     else msg
 }
 
+##' @return combination for  all.equal() functions in ./Matrix.R & ./sparseMatrix.R
+.a.e.comb <- function(msg, r) {
+    if((is.null(msg) || isTRUE(msg)) & (r.ok <- isTRUE(r))) TRUE
+    else c(if(!isTRUE(msg)) msg, if(!r.ok) r)
+}
 
 
 ## chol() via "dpoMatrix"
+## This will only be called for *dense* matrices
 cholMat <- function(x, pivot = FALSE, ...) {
-    ## This will only be called for *dense* matrices
-    px <- as(x, if(length(x@x) < prod(dim(x))) ## packed
-	     "dppMatrix" else "dpoMatrix")
-    if (isTRUE(validObject(px, test=TRUE))) chol(px, pivot, ...)
+    packed <- length(x@x) < prod(dim(x)) ## is it packed?
+    nmCh <- if(packed) "pCholesky" else "Cholesky"
+    if(!is.null(ch <- x@factors[[nmCh]]))
+	return(ch) ## use the cache
+    px <- as(x, if(packed) "dppMatrix" else "dpoMatrix")
+    if (isTRUE(validObject(px, test=TRUE))) ## 'pivot' is not used for dpoMatrix
+	.set.factors(x, nmCh, chol(px, pivot, ...))
     else stop("'x' is not positive definite -- chol() undefined.")
 }
 
@@ -371,7 +390,7 @@ emptyColnames <- function(x, msg.if.not.empty = FALSE)
        is.character(cn <- dn[[2]]) && any(cn != "")) {
 	lc <- length(cn)
 	message(if(lc > 3)
-		gettextf("   [[ suppressing %d column names %s... ]]", nc,
+		gettextf("   [[ suppressing %d column names %s ... ]]", nc,
 			 paste(sQuote(cn[1:3]), collapse = ", "))
 		else
 		gettextf("   [[ suppressing %d column names %s ]]", nc,
@@ -382,6 +401,15 @@ emptyColnames <- function(x, msg.if.not.empty = FALSE)
     x
 }
 
+
+## The i-th unit vector  e[1:n] with e[j] = \delta_{i,j}
+## .E.i.log <- function(i,n)  i == (1:n)
+## .E.i <- function(i,n)
+##     r <- numeric(n)
+##     r[i] <- 1.
+##     r
+## }
+
 idiag <- function(n, p=n)
 {
     ## Purpose: diag() returning  *integer*
@@ -510,7 +538,7 @@ nnzSparse <- function(x, cl = class(x), cld = getClassDef(cl))
 	length(x@i)
     else if(extends(cld, "RsparseMatrix"))
 	length(x@j)
-    else if(extends(cld, "pMatrix"))	# is "sparse" too
+    else if(extends(cld, "indMatrix"))	# is "sparse" too
 	x@Dim[1]
     else stop(gettext("'x' must be \"sparseMatrix\""), domain=NA)
 }
@@ -524,7 +552,7 @@ non0.i <- function(M, cM = class(M), uniqT=TRUE) {
 	if(uniqT && is_not_uniqT(M))
 	    .Call(compressed_non_0_ij, as(M,"CsparseMatrix"), TRUE)
 	else cbind(M@i, M@j)
-    } else if(extends(cM, "pMatrix")) {
+    } else if(extends(cM, "indMatrix")) {
 	cbind(seq_len(nrow(M)), M@perm) - 1L
     } else { ## C* or R*
 	.Call(compressed_non_0_ij, M, extends(cM, "CsparseMatrix"))
@@ -768,7 +796,7 @@ tT2gT <- function(x, cl = class(x), toClass, cld = getClassDef(cl)) {
 ## Fast very special one
 ## .gT2tC <- function(x, uplo, diag) .Call(Tsparse_to_tCsparse, x, uplo, diag)
 
-gT2tT <- function(x, uplo, diag, cl = class(x), toClass,
+gT2tT <- function(x, uplo, diag, toClass,
 		  do.n = extends(toClass, "nMatrix"))
 {
     ## coerce *gTMatrix to *tTMatrix {general -> triangular}
@@ -790,16 +818,35 @@ gT2tT <- function(x, uplo, diag, cl = class(x), toClass,
 	    x = x@x[sel], Dim = x@Dim, Dimnames = x@Dimnames)
 }
 
-check.gT2tT <- function(from, cl = MatrixClass(class(from)),
-			toClass, do.n = extends(toClass, "nMatrix")) {
+check.gT2tT <- function(from, toClass, do.n = extends(toClass, "nMatrix")) {
     if(isTr <- isTriangular(from)) {
-        force(cl)
 	gT2tT(from, uplo = .if.NULL(attr(isTr, "kind"), "U"),
 	      diag = "N", ## improve: also test for unit diagonal
-	      cl = cl, toClass = toClass, do.n = do.n)
+	      toClass = toClass, do.n = do.n)
     } else stop("not a triangular matrix")
 }
 
+gT2sT <- function(x, toClass, do.n = extends(toClass, "nMatrix")) {
+    upper <- x@i <= x@j
+    i <- x@i[upper]
+    j <- x@j[upper]
+    if(do.n) ## no 'x' slot
+	new("nsTMatrix", Dim = x@Dim, Dimnames = x@Dimnames,
+	    i = i, j = j, uplo = "U")
+    else
+	new(toClass, Dim = x@Dim, Dimnames = x@Dimnames,
+	    i = i, j = j, x = x@x[upper], uplo = "U")
+}
+
+check.gT2sT <- function(x, toClass, do.n = extends(toClass, "nMatrix"))
+{
+    if(isSymmetric(x))
+        gT2sT(x, toClass, do.n)
+    else
+	stop("not a symmetric matrix; consider forceSymmetric() or symmpart()")
+}
+
+
 if(FALSE)# unused
 l2d_meth <- function(x) {
     cl <- MatrixClass(class(x))
@@ -810,7 +857,7 @@ l2d_meth <- function(x) {
 .M.kind <- function(x, clx = class(x)) {
     ## 'clx': class() *or* class definition of x
     if(is.matrix(x) || is.atomic(x)) { ## 'old style' matrix or vector
-	if     (is.numeric(x)) "d" ## also for 'integer' --> see .V.kind()
+	if     (is.numeric(x)) "d" ## also for integer: see .V.kind(), .M.kindC()
 	else if(is.logical(x)) "l" ## FIXME ? "n" if no NA ??
 	else if(is.complex(x)) "z"
 	else stop(gettextf("not yet implemented for matrix with typeof %s",
@@ -841,7 +888,7 @@ l2d_meth <- function(x) {
     else if(extends(clx, "dMatrix")) "d"
     else if(extends(clx, "nMatrix")) "n"
     else if(extends(clx, "lMatrix")) "l"
-    else if(extends(clx, "pMatrix")) "n" # permutation -> pattern
+    else if(extends(clx, "indMatrix")) "n" # permutation -> pattern
     else if(extends(clx, "zMatrix")) "z"
     else if(extends(clx, "iMatrix")) "i"
     else stop(gettextf(" not yet implemented for %s", clx@className),
@@ -899,7 +946,7 @@ class2 <- function(cl, kind = "l", do.sub = TRUE) {
 geClass <- function(x) {
     if     (is(x, "dMatrix")) "dgeMatrix"
     else if(is(x, "lMatrix")) "lgeMatrix"
-    else if(is(x, "nMatrix") || is(x, "pMatrix")) "ngeMatrix"
+    else if(is(x, "nMatrix") || is(x, "indMatrix")) "ngeMatrix"
     else if(is(x, "zMatrix")) "zgeMatrix"
     else stop(gettextf("general Matrix class not yet implemented for %s",
 		       dQuote(class(x))), domain = NA)
@@ -1135,16 +1182,20 @@ isTriC <- function(object, upper = NA) {
     }
 }
 
+
+## When the matrix is known to be [n x n] aka "square"
+## (need "vector-indexing" work for 'M'):
+.is.diagonal.sq.matrix <- function(M, n = dim(M)[1L])
+    all0(M[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
+
 .is.diagonal <- function(object) {
     ## "matrix" or "denseMatrix" (but not "diagonalMatrix")
     d <- dim(object)
-    if(d[1] != (n <- d[2])) FALSE
-    else if(is.matrix(object))
-        ## requires that "vector-indexing" works for 'object' :
-        all0(object[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+    if(d[1L] != (n <- d[2L])) FALSE
+    else if(is.matrix(object)) .is.diagonal.sq.matrix(object, n)
     else ## "denseMatrix" -- packed or unpacked
         if(is(object, "generalMatrix")) # "dge", "lge", ...
-            all0(object@x[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+	    .is.diagonal.sq.matrix(object@x, n)
         else { ## "dense" but not {diag, general}, i.e. triangular or symmetric:
             ## -> has 'uplo'  differentiate between packed and unpacked
 
@@ -1156,7 +1207,7 @@ isTriC <- function(object, upper = NA) {
             }
 
 ### very cheap workaround
-	    all0(as.matrix(object)[rep(c(FALSE, rep.int(TRUE,n)), length = n^2)])
+	    all0(as.matrix(object)[rep_len(c(FALSE, rep.int(TRUE,n)), n^2)])
         }
 }
 
@@ -1252,13 +1303,16 @@ diagN2U <- function(x, cl = getClassDef(class(x)), checkDense = FALSE)
 	.Call(Csparse_diagN2U, as(x, "CsparseMatrix"))
 }
 
+.dgC.0.factors <- function(x)
+    if(!length(x@factors)) x else { x@factors <- list() ; x }
 .as.dgC.0.factors <- function(x) {
     if(!is(x, "dgCMatrix"))
 	as(x, "dgCMatrix") # will not have 'factors'
     else ## dgCMatrix
-	if(!length(x@factors)) x else { x@factors <- list() ; x }
+	.dgC.0.factors(x)
 }
 
+.set.factors <- function(x, name, value) .Call(R_set_factors, x, value, name)
 
 ### Fast, much simplified version of tapply()
 tapply1 <- function (X, INDEX, FUN = NULL, ..., simplify = TRUE) {
