@@ -83,6 +83,7 @@ setMethod("mean", signature(x = "Matrix"),
       })
 
 
+## for non-"sparseMatrix" :
 setMethod("cov2cor", signature(V = "Matrix"),
 	  function(V) { ## was as(cov2cor(as(V, "matrix")), "dpoMatrix"))
 	      r <- V
@@ -119,6 +120,10 @@ setMethod("symmpart", signature(x = "matrix"), function(x) (x + t(x))/2)
 setMethod("skewpart", signature(x = "matrix"), function(x) (x - t(x))/2)
 
 
+if(getRversion() > "3.1.0")
+## NB: "nsparseMatrix" has extra method
+setMethod("anyNA", signature(x = "xMatrix"),
+	  function(x) anyNA(x@x))
 
 
 setMethod("dim", signature(x = "Matrix"),
@@ -299,7 +304,7 @@ setMethod("chol2inv", signature(x = "sparseMatrix"),
 	      tcrossprod(solve(as(x,"triangularMatrix")))
 	  })
 
-## There are special sparse methods; this is a "fall back":
+## There are special sparse methods in  ./kronecker.R  ; this is a "fall back":
 setMethod("kronecker", signature(X = "Matrix", Y = "ANY",
 				 FUN = "ANY", make.dimnames = "ANY"),
 	  function(X, Y, FUN, make.dimnames, ...) {
@@ -319,8 +324,7 @@ setMethod("determinant", signature(x = "Matrix", logarithm = "missing"),
           function(x, logarithm, ...)
           determinant(x, logarithm = TRUE, ...))
 
-## Gives  "The following object(s) are masked from package:base :   det"
-## but is still ``the Right Thing'' to do :
+## The ``Right Thing'' to do :
 ## base::det() calls [base::]determinant();
 ## our det() should call our determinant() :
 det <- base::det
@@ -359,7 +363,7 @@ setMethod("rcond", signature(x = "ANY", norm = "missing"),
 	  function(x, norm, ...) rcond(x, norm = "O", ...))
 
 setMethod("lu", "matrix", function(x, warnSing = TRUE, ...)
-	  lu(as(x, "dgeMatrix"), warnSing=warnSing, ...))
+	  lu(..2dge(x), warnSing=warnSing, ...))
 
 
 
@@ -373,11 +377,10 @@ all.equal_Mat <- function(target, current, check.attributes = TRUE,
 {
     msg <- attr.all_Mat(target, current, check.attributes=check.attributes,
                         factorsCheck=factorsCheck, ...)
-    if(is.list(msg)) return(msg[[1]])
-    ## else
-    r <- all.equal_num(as.vector(target), as.vector(current),
-                       check.attributes=check.attributes, ...)
-    if(is.null(msg) & (r.ok <- isTRUE(r))) TRUE else c(msg, if(!r.ok) r)
+    if(is.list(msg)) msg[[1]]
+    else .a.e.comb(msg,
+		   all.equal_num(as.vector(target), as.vector(current),
+				 check.attributes=check.attributes, ...))
 }
 ## The all.equal() methods for dense matrices (and fallback):
 setMethod("all.equal", c(target = "Matrix", current = "Matrix"),
@@ -503,7 +506,7 @@ Summary.np <- function(x, ..., na.rm) {
 ##
 setMethod("Summary", signature(x = "lMatrix", na.rm = "ANY"), Summary.l)
 setMethod("Summary", signature(x = "nMatrix", na.rm = "ANY"), Summary.np)
-setMethod("Summary", signature(x = "pMatrix", na.rm = "ANY"), Summary.np)
+setMethod("Summary", signature(x = "indMatrix", na.rm = "ANY"), Summary.np)
 
 ## NOTE:  "&" and "|"  are now in group "Logic" c "Ops" --> ./Ops.R
 ##        "!" is in ./not.R
@@ -593,10 +596,12 @@ setMethod("[", signature(x = "Matrix", i = "ANY", j = "ANY", drop = "ANY"),
 for(ii in c("lMatrix", "logical"))
     setMethod("[", signature(x = "Matrix", i = ii, j = "missing", drop = "missing"),
 	      .M.sub.i.logical)
+rm(ii)
 
 
+##' x[ ij ]  where ij is (i,j) 2-column matrix
+##' @note only called from  .M.sub.i.2col(x, i) below
 subset.ij <- function(x, ij) {
-    ## x[ ij ]  where ij is (i,j) 2-column matrix
     m <- nrow(ij)
     if(m > 3) {
         cld <- getClassDef(class(x))
@@ -626,13 +631,12 @@ subset.ij <- function(x, ij) {
 
 	    m1 <- .Call(m_encodeInd, ij.x, di, checkBounds = FALSE)
             m2 <- .Call(m_encodeInd, ij -1L, di, checkBounds = TRUE)
-	    mi <- match(m1, m2, nomatch=0)
-	    mmi <- mi != 0
-	    ## Result:
+	    mi <- match(m2, m1, nomatch=0)
+	    mmi <- mi != 0L ## == (m2 %in% m1)
+	    ## Result: all FALSE or 0  apart from where we match non-zero entries
 	    ans <- vector(mode = .type.kind[.M.kindC(cld)], length = m)
 	    ## those that are *not* zero:
-	    ans[mi[mmi]] <-
-		if(extends(cld, "nsparseMatrix")) TRUE else x@x[mmi]
+	    ans[mmi] <- if(extends(cld, "nsparseMatrix")) TRUE else x@x[mi[mmi]]
 	    if(any(ina <- is.na(m2))) # has one or two NA in that (i,j) row
 		is.na(ans) <- ina
 	    ans
@@ -656,21 +660,20 @@ subset.ij <- function(x, ij) {
 .M.sub.i.2col <- function (x, i, j, ..., drop)
 {
     nA <- nargs()
-    if(nA == 2) { ##  M [ cbind(ii,jj) ] or M [ <logical matrix> ]
-	if(!is.integer(nc <- ncol(i)))
-	    stop(".M.sub.i.2col(): 'i' has no integer column number;\n should never happen; please report")
-	if(is.logical(i))
-	    return(.M.sub.i.logical(x, i=i)) # call with 2 args!
-	else if(!is.numeric(i) || nc != 2)
-	    stop("such indexing must be by logical or 2-column numeric matrix")
-	m <- nrow(i)
-        if(m == 0) return(vector(mode = .type.kind[.M.kind(x)]))
-        ## else
-        subset.ij(x, i)
+    if(nA != 2)
+        stop(domain=NA, gettextf(
+            "nargs() = %d.  Extraneous illegal arguments inside '[ .. ]' (i.2col)?", nA))
+    ## else: (nA == 2):	 M [ cbind(ii,jj) ] or M [ <logical matrix> ]
+    if(!is.integer(nc <- ncol(i)))
+        stop(".M.sub.i.2col(): 'i' has no integer column number;\n should never happen; please report")
+    if(is.logical(i))
+        return(.M.sub.i.logical(x, i=i)) # call with 2 args!
+    else if(!is.numeric(i) || nc != 2)
+        stop("such indexing must be by logical or 2-column numeric matrix")
+    if(!nrow(i)) return(vector(mode = .type.kind[.M.kind(x)]))
+    ## else
+    subset.ij(x, i)
 
-    } else stop(gettextf(
-		"nargs() = %d.  Extraneous illegal arguments inside '[ .. ]' (i.2col)?",
-			 nA), domain=NA)
 }
 setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing"),# drop="ANY"
 	  .M.sub.i.2col)
@@ -683,7 +686,8 @@ setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing", drop="missin
 
 ## A[ ij ] <- value,  where ij is (i,j) 2-column matrix :
 ## ----------------
-## The cheap general method --- FIXME: provide special ones; done for Tsparse..
+## The cheap general method, now only used for "pMatrix","indMatrix"
+## sparse all use  .TM.repl.i.mat()
 ## NOTE:  need '...' below such that setMethod() does
 ##	  not use .local() such that nargs() will work correctly:
 .M.repl.i.2col <- function (x, i, j, ..., value)
@@ -712,7 +716,7 @@ setMethod("[", signature(x = "Matrix", i = "matrix", j = "missing", drop="missin
 	if(length(value) > 0 && m %% length(value) != 0)
 	    warning("number of items to replace is not a multiple of replacement length")
 	## recycle:
-	value <- rep(value, length = m)
+	value <- rep_len(value, m)
 	i1 <- i[,1]
 	i2 <- i[,2]
 	if(m > 2)
