@@ -40,21 +40,11 @@ SEXP compressed_to_TMatrix(SEXP x, SEXP colP)
      * Csparse_to_Tsparse() in ./Csparse.c ; maybe should simply write
      * an  as_cholmod_Rsparse() function and then do "as there" ...*/
     SEXP indSym = col ? Matrix_iSym : Matrix_jSym,
-	ans,	indP = GET_SLOT(x, indSym),
+	ans, indP = GET_SLOT(x, indSym),
 	pP = GET_SLOT(x, Matrix_pSym);
     int npt = length(pP) - 1;
     char *ncl = strdup(class_P(x));
-    char *valid[] = {
-	"dgCMatrix", "dsCMatrix", "dtCMatrix", /* 0: 0:2 */
-	"lgCMatrix", "lsCMatrix", "ltCMatrix", /* 1: 3:5 */
-	"ngCMatrix", "nsCMatrix", "ntCMatrix", /* 2: 6:8 */
-	"zgCMatrix", "zsCMatrix", "ztCMatrix", /* 3: 9:11 */
-
-	"dgRMatrix", "dsRMatrix", "dtRMatrix", /* 4: 12:14 */
-	"lgRMatrix", "lsRMatrix", "ltRMatrix", /* 5: 15:17 */
-	"ngRMatrix", "nsRMatrix", "ntRMatrix", /* 6: 18:20 */
-	"zgRMatrix", "zsRMatrix", "ztRMatrix", /* 7: 21:23 */
-	""};
+    static const char *valid[] = { MATRIX_VALID_Csparse, MATRIX_VALID_Rsparse, ""};
     int ctype = Matrix_check_class_etc(x, valid);
 
     if (ctype < 0)
@@ -86,12 +76,7 @@ SEXP R_to_CMatrix(SEXP x)
 {
     SEXP ans, tri = PROTECT(allocVector(LGLSXP, 1));
     char *ncl = strdup(class_P(x));
-    char *valid[] = {
-	"dgRMatrix", "dsRMatrix", "dtRMatrix",
-	"lgRMatrix", "lsRMatrix", "ltRMatrix",
-	"ngRMatrix", "nsRMatrix", "ntRMatrix",
-	"zgRMatrix", "zsRMatrix", "ztRMatrix",
-	""};
+    static const char *valid[] = { MATRIX_VALID_Rsparse, ""};
     int ctype = Matrix_check_class_etc(x, valid);
     int *x_dims = INTEGER(GET_SLOT(x, Matrix_DimSym)), *a_dims;
     PROTECT_INDEX ipx;
@@ -128,6 +113,10 @@ SEXP R_to_CMatrix(SEXP x)
     return ans;
 }
 
+/** Return a 2 column matrix  '' cbind(i, j) ''  of 0-origin index vectors (i,j)
+ *  which entirely correspond to the (i,j) slots of
+ *  as(x, "TsparseMatrix") :
+ */
 SEXP compressed_non_0_ij(SEXP x, SEXP colP)
 {
     int col = asLogical(colP); /* 1 if "C"olumn compressed;  0 if "R"ow */
@@ -135,13 +124,13 @@ SEXP compressed_non_0_ij(SEXP x, SEXP colP)
     SEXP indP = GET_SLOT(x, indSym),
 	pP = GET_SLOT(x, Matrix_pSym);
     int i, *ij;
-    int ncol = INTEGER(GET_SLOT(x, Matrix_DimSym))[1],
-	n_el = INTEGER(pP)[ncol]; /* is only == length(indP), if the
-				     i-slot is not over-allocated */
+    int nouter = INTEGER(GET_SLOT(x, Matrix_DimSym))[col ? 1 : 0],
+	n_el   = INTEGER(pP)[nouter]; /* is only == length(indP), if the
+				     inner slot is not over-allocated */
 
     ij = INTEGER(ans = PROTECT(allocMatrix(INTSXP, n_el, 2)));
     /* expand the compressed margin to 'i' or 'j' : */
-    expand_cmprPt(ncol, INTEGER(pP), &ij[col ? n_el : 0]);
+    expand_cmprPt(nouter, INTEGER(pP), &ij[col ? n_el : 0]);
     /* and copy the other one: */
     if (col)
 	for(i = 0; i < n_el; i++)
@@ -183,7 +172,7 @@ SEXP dgCMatrix_qrsol(SEXP x, SEXP y, SEXP ord)
     int order = INTEGER(ord)[0];
 #ifdef _not_yet_do_FIXME__
     const char *nms[] = {"L", "coef", "Xty", "resid", ""};
-    SEXP ans = PROTECT(Matrix_make_named(VECSXP, nms));
+    SEXP ans = PROTECT(Rf_mkNamed(VECSXP, nms));
 #endif
     R_CheckStack();
 
@@ -211,46 +200,41 @@ SEXP dgCMatrix_qrsol(SEXP x, SEXP y, SEXP ord)
 	error(_("cs_qrsol() failed inside dgCMatrix_qrsol()"));
 
     /* Solution is only in the first part of ycp -- cut its length back to n : */
-    {
-	SEXP nms = getAttrib(ycp, R_NamesSymbol);
-	SETLENGTH(ycp, xc->n);
-	if(nms != R_NilValue) {
-	    SETLENGTH(nms, xc->n);
-	    setAttrib(ycp, R_NamesSymbol, nms);
-	}
-    }
+    ycp = lengthgets(ycp, (R_len_t) xc->n);
+
     UNPROTECT(1);
     return ycp;
 }
 
-/* Modified version of Tim Davis's cs_qr_mex.c file for MATLAB */
+// Modified version of Tim Davis's cs_qr_mex.c file for MATLAB (in CSparse)
+//  Usage: [V,beta,p,R,q] = cs_qr(A) ;
 SEXP dgCMatrix_QR(SEXP Ap, SEXP order)
 {
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("sparseQR")));
     CSP A = AS_CSP__(Ap), D;
-    css *S;
-    csn *N;
+    int io = INTEGER(order)[0];
+    Rboolean verbose = (io < 0);// verbose=TRUE, encoded with negative 'order'
     int m = A->m, n = A->n, ord = asLogical(order) ? 3 : 0, *p;
-    int *dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     R_CheckStack();
 
     if (m < n) error(_("A must have #{rows} >= #{columns}")) ;
+    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("sparseQR")));
+    int *dims = INTEGER(ALLOC_SLOT(ans, Matrix_DimSym, INTSXP, 2));
     dims[0] = m; dims[1] = n;
-    S = cs_sqr(ord, A, 1);	/* symbolic QR ordering & analysis*/
+    css *S = cs_sqr(ord, A, 1);	/* symbolic QR ordering & analysis*/
     if (!S) error(_("cs_sqr failed"));
-    N = cs_qr(A, S);		/* numeric QR factorization */
+    if(verbose && S->m2 > m) // in ./cs.h , m2 := # of rows for QR, after adding fictitious rows
+	Rprintf("Symbolic QR(): Matrix structurally rank deficient (m2-m = %d)\n",
+		S->m2 - m);
+    csn *N = cs_qr(A, S);		/* numeric QR factorization */
     if (!N) error(_("cs_qr failed")) ;
     cs_dropzeros(N->L);		/* drop zeros from V and sort */
-    D = cs_transpose(N->L, 1);
-    cs_spfree(N->L);
-    N->L = cs_transpose(D, 1);
-    cs_spfree(D);
+    D = cs_transpose(N->L, 1); cs_spfree(N->L);
+    N->L = cs_transpose(D, 1); cs_spfree(D);
     cs_dropzeros(N->U);		/* drop zeros from R and sort */
-    D = cs_transpose(N->U, 1);
-    cs_spfree(N->U) ;
-    N->U = cs_transpose(D, 1);
-    cs_spfree(D);
+    D = cs_transpose(N->U, 1); cs_spfree(N->U) ;
+    N->U = cs_transpose(D, 1); cs_spfree(D);
     m = N->L->m;		/* m may be larger now */
+    // MM: m := S->m2  also counting the ficticious rows (Tim Davis, p.72, 74f)
     p = cs_pinv(S->pinv, m);	/* p = pinv' */
     SET_SLOT(ans, install("V"),
 	     Matrix_cs_to_SEXP(N->L, "dgCMatrix", 0));
@@ -304,7 +288,7 @@ SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
     if ((rank = SuiteSparseQR_C_QR(asInteger(ordering),
 				   asReal(tol),/* originally had SPQR_DEFAULT_TOL */
 				   (UF_long)asInteger(econ),/* originally had 0 */
-				   A, &Q, &R, &E, &c)) == -1)
+				   A, &Q, &R, &E, &cl)) == -1)
 	error(_("SuiteSparseQR_C_QR returned an error code"));
 
     SET_SLOT(ans, Matrix_DimSym, duplicate(GET_SLOT(Ap, Matrix_DimSym)));
@@ -319,18 +303,17 @@ SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
 /* 		   chm_sparse_to_SEXP(R, 0, 0, 0, "", R_NilValue)); */
     SET_SLOT(ans, install("R"),
 	     chm_sparse_to_SEXP(R, 0, 0, 0, "", R_NilValue));
-    cholmod_l_free_sparse(&R, &c);
-    cholmod_l_free_sparse(&Q, &c);
+    cholmod_free_sparse(&Al, &cl);
+    cholmod_free_sparse(&R, &cl);
+    cholmod_free_sparse(&Q, &cl);
     if (E) {
-	/* FIXME when UF_long != int  ?! */
-	Memcpy(INTEGER(ALLOC_SLOT(ans, Matrix_pSym, /* "p" */
-				  INTSXP, A->ncol)), E, A->ncol);
-/* 	SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, A->ncol)); */
-/* 	Memcpy(INTEGER(VECTOR_ELT(ans, 2)), E, A->ncol); */
+	int *Er;
+	SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, A->ncol));
+	Er = INTEGER(VECTOR_ELT(ans, 2));
+	for (int i = 0; i < A->ncol; i++) Er[i] = (int) E[i];
 	Free(E);
-    } /* else SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, 0)); */
-
-    SET_SLOT(ans, install("rank"), ScalarInteger((int)rank));
+    } else SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, 0));
+    SET_VECTOR_ELT(ans, 3, ScalarInteger((int)rank));
     UNPROTECT(1);
     return ans;
 }
@@ -340,6 +323,7 @@ SEXP dgCMatrix_SPQR(SEXP Ap, SEXP ordering, SEXP econ, SEXP tol)
 /* Modified version of Tim Davis's cs_lu_mex.c file for MATLAB */
 void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing)
 {
+    // (order, tol) == (1, 1) by default, when called from R.
     SEXP ans;
     css *S;
     csn *N;
@@ -354,14 +338,14 @@ void install_lu(SEXP Ap, int order, double tol, Rboolean err_sing)
 	order = (tol == 1) ? 2	/* amd(S'*S) w/dense rows or I */
 	    : 1;		/* amd (A+A'), or natural */
     }
-    S = cs_sqr(order, A, 0);	/* symbolic ordering, no QR bound */
+    S = cs_sqr(order, A, /*qr = */ 0);	/* symbolic ordering */
     N = cs_lu(A, S, tol);	/* numeric factorization */
     if (!N) {
 	if(err_sing)
 	    error(_("cs_lu(A) failed: near-singular A (or out of memory)"));
 	else {
 	    /* No warning: The useR should be careful :
-	     * Put  FALSE  into  "LU" factor */
+	     * Put  NA  into  "LU" factor */
 	    set_factors(Ap, ScalarLogical(NA_LOGICAL), "LU");
 	    return;
 	}
@@ -413,8 +397,21 @@ SEXP dgCMatrix_LU(SEXP Ap, SEXP orderp, SEXP tolp, SEXP error_on_sing)
     return get_factors(Ap, "LU");
 }
 
-SEXP dgCMatrix_matrix_solve(SEXP Ap, SEXP b)
+SEXP dgCMatrix_matrix_solve(SEXP Ap, SEXP b, SEXP give_sparse)
 {
+    Rboolean sparse = asLogical(give_sparse);
+    if(sparse) {
+	// FIXME: implement this
+	error(_("dgCMatrix_matrix_solve(.., sparse=TRUE) not yet implemented"));
+
+	/* Idea: in the  for(j = 0; j < nrhs ..) loop below, build the *sparse* result matrix
+	 * ----- *column* wise -- which is perfect for dgCMatrix
+	 * --> build (i,p,x) slots "increasingly" [well, allocate in batches ..]
+	 *
+	 * --> maybe first a protoype in R
+	 */
+
+    }
     SEXP ans = PROTECT(dup_mMatrix_as_dgeMatrix(b)),
 	lu, qslot;
     CSP L, U;
@@ -425,7 +422,7 @@ SEXP dgCMatrix_matrix_solve(SEXP Ap, SEXP b)
     R_CheckStack();
 
     if (isNull(lu = get_factors(Ap, "LU"))) {
-	install_lu(Ap, 1, 1.0, /* err_sing = */ TRUE);
+	install_lu(Ap, /* order = */ 1, /* tol = */ 1.0, /* err_sing = */ TRUE);
 	lu = get_factors(Ap, "LU");
     }
     qslot = GET_SLOT(lu, install("q"));
@@ -442,7 +439,7 @@ SEXP dgCMatrix_matrix_solve(SEXP Ap, SEXP b)
 	cs_pvec(p, ax + j * n, x, n);  /* x = b(p) */
 	cs_lsolve(L, x);	       /* x = L\x */
 	cs_usolve(U, x);	       /* x = U\x */
-	if (q)			       /* b(q) = x */
+	if (q)			       /* r(q) = x , hence r = Q' U{^-1} L{^-1} P b = A^{-1} b */
 	    cs_ipvec(q, x, ax + j * n, n);
 	else
 	    Memcpy(ax + j * n, x, n);
@@ -464,26 +461,26 @@ SEXP dgCMatrix_cholsol(SEXP x, SEXP y)
     int n = cx->ncol;/* #{obs.} {x = t(X) !} */
     double one[] = {1,0}, zero[] = {0,0}, neg1[] = {-1,0};
     const char *nms[] = {"L", "coef", "Xty", "resid", ""};
-    SEXP ans = PROTECT(Matrix_make_named(VECSXP, nms));
+    SEXP ans = PROTECT(Rf_mkNamed(VECSXP, nms));
     R_CheckStack();
 
     if (n < cx->nrow || n <= 0)
 	error(_("dgCMatrix_cholsol requires a 'short, wide' rectangular matrix"));
     if (cy->nrow != n)
 	error(_("Dimensions of system to be solved are inconsistent"));
-    rhs = cholmod_l_allocate_dense(cx->nrow, 1, cx->nrow, CHOLMOD_REAL, &c);
+    rhs = cholmod_allocate_dense(cx->nrow, 1, cx->nrow, CHOLMOD_REAL, &c);
     /* cholmod_sdmult(A, transp, alpha, beta, X, Y, &c):
      *		Y := alpha*(A*X) + beta*Y or alpha*(A'*X) + beta*Y ;
      * here: rhs := 1 * x %*% y + 0 =  x %*% y =  X'y  */
-    if (!(cholmod_l_sdmult(cx, 0 /* trans */, one, zero, cy, rhs, &c)))
-	error(_("cholmod_l_sdmult error (rhs)"));
-    L = cholmod_l_analyze(cx, &c);
-    if (!cholmod_l_factorize(cx, L, &c))
-	error(_("cholmod_l_factorize failed: status %d, minor %d from ncol %d"),
+    if (!(cholmod_sdmult(cx, 0 /* trans */, one, zero, cy, rhs, &c)))
+	error(_("cholmod_sdmult error (rhs)"));
+    L = cholmod_analyze(cx, &c);
+    if (!cholmod_factorize(cx, L, &c))
+	error(_("cholmod_factorize failed: status %d, minor %d from ncol %d"),
 	      c.status, L->minor, L->n);
 /* FIXME: Do this in stages so an "effects" vector can be calculated */
-    if (!(cAns = cholmod_l_solve(CHOLMOD_A, L, rhs, &c)))
-	error(_("cholmod_l_solve (CHOLMOD_A) failed: status %d, minor %d from ncol %d"),
+    if (!(cAns = cholmod_solve(CHOLMOD_A, L, rhs, &c)))
+	error(_("cholmod_solve (CHOLMOD_A) failed: status %d, minor %d from ncol %d"),
 	      c.status, L->minor, L->n);
     /* L : */
     SET_VECTOR_ELT(ans, 0, chm_factor_to_SEXP(L, 0));
@@ -495,20 +492,19 @@ SEXP dgCMatrix_cholsol(SEXP x, SEXP y)
     SET_VECTOR_ELT(ans, 2, allocVector(REALSXP, cx->nrow));
     Memcpy(REAL(VECTOR_ELT(ans, 2)), (double*)(rhs->x), cx->nrow);
     /* resid := y */
-    resid = cholmod_l_copy_dense(cy, &c);
+    resid = cholmod_copy_dense(cy, &c);
     /* cholmod_sdmult(A, transp, alp, bet, X, Y, &c):
      *		Y := alp*(A*X) + bet*Y or alp*(A'*X) + beta*Y ;
      * here: resid := -1 * x' %*% coef + 1 * y = y - X %*% coef  */
-    if (!(cholmod_l_sdmult(cx, 1/* trans */, neg1, one, cAns, resid, &c)))
-	error(_("cholmod_l_sdmult error (resid)"));
+    if (!(cholmod_sdmult(cx, 1/* trans */, neg1, one, cAns, resid, &c)))
+	error(_("cholmod_sdmult error (resid)"));
     /* FIXME: for multivariate case, i.e. resid  *matrix* with > 1 column ! */
     SET_VECTOR_ELT(ans, 3, allocVector(REALSXP, n));
     Memcpy(REAL(VECTOR_ELT(ans, 3)), (double*)(resid->x), n);
 
-    cholmod_l_free_factor(&L, &c);
-    cholmod_l_free_dense(&rhs, &c);
-    cholmod_l_free_dense(&resid, &c);
-    cholmod_l_free_dense(&cAns, &c);
+    cholmod_free_factor(&L, &c);
+    cholmod_free_dense(&rhs, &c);
+    cholmod_free_dense(&cAns, &c);
     UNPROTECT(1);
     return ans;
 }
